; elminates screen garbage when an AutoLISP error happens
(defun *error* (msg)
   (princ "\nerror: ")
   (princ msg)
   (terpri)
   (restore)
)

; saves enviroment
(defun store ()

   (setvar "UserI1" (getvar "OsMode"))
   (setvar "UserI2" (getvar "OrthoMode"))
   (setvar "UserI3" (getvar "Aperture"))
   (setvar "UserI4" (getvar "Pickbox"))
   (setvar "UserI5" (getvar "BlipMode"))
)

(defun argh () (princ "Invalid input..."))

; restores enviroment
(defun restore ()
   (setvar "CmdEcho" 1)
   (setvar "OsMode"    (getvar "UserI1"))
   (setvar "OrthoMode" (getvar "UserI2"))
   (setvar "Aperture"  (getvar "UserI3"))
   (setvar "Pickbox"   (getvar "UserI4"))
   (setvar "BlipMode"  (getvar "UserI5"))
)

; Wall1 - draws line(s) offset from user line (with corners FILLETed)
(defun c:WALL1 (/ pt1 pt2 ss ang rl)
   (store)
   (setvar "CmdEcho" 0)
   (setvar "FilletRad" 0)
   (setq temp nil)
   (while (not temp)
       (setq temp (getdist (strcat "\nOffset distance"
         (if d (strcat " <" (rtos d (getvar "lunits")
         (getvar "luprec")) ">") "") ": ")))
       (cond
            ((> temp 0) (setq d temp))
            ((and (not temp) d) (setq temp T))
            (T (progn
               (setq temp nil)
               (princ "Value must be positive and nonzero")
               )
            )
       )
   )
   (while (not (setq pt1 (getpoint "\nFrom point: "))) (argh))
   (while (not (setq pt2 (getpoint pt1 "\nTo point: "))) (argh))
   (command "LINE" pt1 pt2 "")
   (while (not (setq rl (getpoint "\nWhich side? "))) (argh))
   (setq rl (- (angle pt1 rl) (setq ang (angle pt1 pt2))))
   (if (< rl 0) (setq rl (+ rl pi pi)))
   (if (< rl pi) (setq rl +) (setq rl -))
   (command "COPY" "L" "" pt1 (polar pt1 (rl ang 1.570796) d))
   (setq ss (ssadd (entlast)))
   (setq pt1 pt2)
   (while (setq pt2 (getpoint pt1 "To point: "))
       (command "LINE" pt1 pt2 "")
       (setq ang (angle pt1 pt2))
       (command "COPY" "L" "" pt1 (polar pt1 (rl ang 1.570796) d))
       (setq ss (ssadd (entlast) ss))
       (command "FILLET" ss)
       (ssdel (ssname ss 0) ss)
       (setq pt1 pt2)
   )
   (restore)
   (command)
)


;Wall2
(defun c:WALL2 (/ pt1 pt2 ss1 ss2 ang)
   (store)
   (setvar "CmdEcho" 0)
   (setvar "FilletRad" 0)
   (setq temp nil)
   (while (not temp)
       (setq temp (getdist (strcat "\nOffset distance"
         (if d (strcat " <" (rtos d (getvar "lunits")
         (getvar "luprec")) ">") "") ": ")))
       (cond
            ((> temp 0) (setq d temp))
            ((and (not temp) d) (setq temp T))
            (T (progn
               (setq temp nil)
               (princ "Value must be positive and nonzero")
               )
            )
       )
   )
   (while (not (setq pt1 (getpoint "\nFrom point: "))) (argh))
   (while (not (setq pt2 (getpoint pt1 "\nTo point: "))) (argh))
   (command "LINE" pt1 pt2 "")
   (setq ang (angle pt1 pt2))
   (command "MOVE" "L" "" pt1 (polar pt1 (- ang 1.570796) (/ d 2)))
   (setq ss1 (ssadd (entlast)))
   (command "COPY" "L" "" pt1 (polar pt1 (+ ang 1.570796) d))
   (setq ss2 (ssadd (entlast)))
   (setq pt1 pt2)
   (while (setq pt2 (getpoint pt1 "To point: "))
      (command "LINE" pt1 pt2 "")
      (setq ang (angle pt1 pt2))
      (command "MOVE" "L" "" pt1 (polar pt1 (- ang 1.570796) (/ d 2)))
      (setq ss1 (ssadd (entlast) ss1))
      (command "COPY" "L" "" pt1 (polar pt1 (+ ang 1.570796) d))
      (setq ss2 (ssadd (entlast) ss2))
      (command "FILLET" ss1)
      (command "FILLET" ss2)
      (ssdel (ssname ss1 0) ss1)
      (ssdel (ssname ss2 0) ss2)
      (setq pt1 pt2)
   )
   (restore)
   (command)
)

;Neat - cleans up wall (parallel line) intersections
(defun c:NEAT (/ pt1 pt2 lst linint intlst i j num ss)
   (store)
   (setvar "CmdEcho" 0)
   (setvar "OsMode" 0)
   (setvar "SnapMode" 0)
   (setvar "PickBox" 1)
   (setvar "FilletRad" 0)
   (while (not (setq pt1 (getpoint "\nFirst corner: "))) (argh))
   (while (not (setq pt2 (getcorner pt1 "\nOther corner: ")))
       (argh)
   )
   (setq ss (ssget "C" pt1 pt2))
   (setq num (sslength ss) i 0)
   (while (< i num)
      (setq ent (entget (ssname ss i)))
      (if (= (cdr (assoc 0 ent)) "LINE")
           (progn
           (setq lst (cons (list (cdr (assoc 10 ent))
                 (cdr (assoc 11 ent))) lst))
           (setq i (1+ i))
           )
           (progn
           (ssdel (ssname ss i) ss)
           (setq num (1- num))
           )
      )
  )
  (setq i 0)
  (while (< i num)
      (setq j 0 intlst nil)
      (while (< j num)
          (if (setq int (inters (car (nth i lst)) (cadr (nth i lst))
               (car (nth j lst)) (cadr (nth j lst)) T))
               (setq intlst (cons int intlst))
          )
          (setq j (1+ j))
       )
       (setq linint (cons intlst linint) i (1+ i))
  )
  (setq i 0 lst nil)
  (while (< i num)
      (if (> (length (setq intlst (nth i linint))) 1)
           (progn
           (setq pt1 (car intlst) pt2 (cadr intlst))
           (command "BREAK" (polar pt1 (angle pt1 pt2)
           (/ (distance pt1 pt2) 2)) "F" pt1 pt2)
           )
           (setq lst (cons (car intlst) lst))
      )
      (setq i (1+ i))
  )
  (setq num (length lst) i 0)
  (while (< i num)
       (command "FILLET" (nth i lst) "@")
       (setq i (1+ i))
  )
  (restore)
  (command)
)

; Special Thanks to Mike Barnes and Rebecca O'Rourke for help doing the
; following routines.

;Opening - generates opening in wall (i.e. parallel lines).
(defun c:OPENING ()
    (store)
    (setvar "CmdEcho" 0)
    (brkwall "opening" "Insertion")
    (restore)
    (command)
)

;Door - generates door in wall.
(defun c:DOOR ()
    (store)
    (setvar "CmdEcho" 0)
    (if (not ang2) (setq ang2 1.570796))
    (brkwall "door" "Hinge" nil nil)
    (setq temp nil)
    (while (not temp)
       (setq temp (getangle (strcat "\nAngle of door <"
          (angtos ang2 (getvar "aunits") (getvar "auprec")) ">: ")))
       (cond
            ((and (> temp 0) (< temp pi)) (setq ang2 temp))
            ((not temp) (setq temp T))
            (T (progn
               (setq temp nil)
               (princ "Angle must be between 0 and 180 degrees.")
               )
            )
       )
    )
    (setq p5 (polar p1 (rl ang 1.570796 ang2) width))
    (command "LINE" p1 p5 "")
    (command "ARC" "C" p1 p3 "A" (rl (* ang2 57.295780)))
    (restore)
    (command)
)

;Window - generates 'exterior' window in wall.
(defun c:WINDOW ()
    (store)
    (setvar "CmdEcho" 0)
    (brkwall "window" "Inside" T nil)
    (setq dist (/ (distance p1 p2) 2))
    (command "LINE" (polar p1 ang dist) (polar p3 ang dist) "")
    (setq temp nil)
    (while (not temp)
        (setq temp (getdist p2 (strcat "\nWidth of ledge"
          (if ledge (strcat " <" (rtos ledge (getvar "lunits")
          (getvar "luprec")) ">") "") ": ")))
        (cond
             ((> temp 0) (setq ledge temp))
             ((and (not temp) ledge) (setq temp T))
             (T (progn
                (setq temp nil)
                (princ "Value must be positive and nonzero")
                )
             )
        )
    )
    (command "PLINE" p2 (polar p2 ang ledge) (polar p4 ang ledge) p4 "")
    (restore)
    (command)
)

;RELITE - generates 'interior' window in wall.
(defun c:RELITE ()
    (store)
    (setvar "CmdEcho" 0)
    (if (equal (setq a (getstring "\nRemove wall? No/<Yes>:")) "")
       (setq a nil)
    )
    (brkwall "window" "Insertion" a a)
    (setq dist (/ (distance p1 p2) 2))
    (command "LINE" (polar p1 ang dist) (polar p3 ang dist) "")
    (restore)
    (command)
)

;used by OPENING, DOOR and WINDOW
(defun brkwall (msg1 msg2 a b)
   (setvar "BlipMode" 1)
   (setvar "OsMode" 512)
   (while (not (setq p1 (getpoint (strcat "\n" msg2 " point on wall: "))))
      (argh)
   )
   (setvar "OsMode" 128)
   (while (not (setq p2 (getpoint p1 "\nTouch opposite side of wall: ")))
      (argh)
   )
   (setvar "BlipMode" 0)
   (setvar "OsMode" 0)
   (setq temp nil)
   (while (not temp)
       (setq temp (getdist p1 (strcat "\nWidth of " msg1
         (if width (strcat " <" (rtos width (getvar "lunits")
         (getvar "luprec")) ">") "") ": ")))
       (cond
            ((> temp 0) (setq width temp))
            ((and (not temp) width) (setq temp T))
            (T (progn
               (setq temp nil)
               (princ "Value must be positive and nonzero")
               )
            )
       )
   )
   (while (not (setq rl (getpoint "\nWhich side? ")))
      (argh)
   )
   (setq rl (- (angle p1 rl) (setq ang (angle p1 p2))))
   (if (< rl 0) (setq rl (+ rl pi pi)))
   (if (< rl pi) (setq rl +) (setq rl -))
   (setq p3 (polar p1 (rl ang 1.570796) width))
   (setq p4 (polar p2 (rl ang 1.570796) width))
   (if a (command "BREAK" p1 p1 "BREAK" p3 p3) (command "BREAK" p1 p3))
   (if b (command "BREAK" p2 p2 "BREAK" p4 p4) (command "BREAK" p2 p4))
   (command "LINE" p1 p2 "")
   (command "LINE" p3 p4 "")
)
(defun Cdate ( / CD)
    (setq CD (rtos (getvar "CDATE") 2 6))
    (strcat (substr CD 5 2) "/" (substr CD 7 2) "/" (substr CD 3 2) " "
            (substr CD 10 2) ":" (substr CD 12 2) ":" (substr CD 14 2)))
;
;
;  Listing 2.  TIMER:   Returns the interval time every other call.
;                       Elapsed time in fractions of a day.
;
(defun Timer ( / ELPS)
   (if (null TMON)
      (setq TMON (getvar "DATE") ELPS nil)  
      (setq ELPS (- (getvar "DATE") TMON) TMON nil))
   ELPS)
;
;
;
(defun c:AutoSAVE ()
   (if (null Tsaved) (setq Tsaved (getvar "DATE")))
   (if 
      (> (* (- (getvar "DATE") Tsaved) 1440.0) 15.0) 
          (progn
             (command "SAVE" "")
             (setq Tsaved (getvar "DATE")))))
;
;
;  Listing 4. TIME:      Returns the time in a string format given  
;                        the Julian time.
;
(defun Time (JDT) ; JDT is Julian Date/Time
    (setq JDT (- JDT (fix JDT))
          HH (* JDT 24.0)
          MM (* (- HH (fix HH)) 60.0)
          SS (* (- MM (fix MM)) 60.0)) 
    (strcat (itoa (fix HH)) ":" 
            (if (< (fix MM) 10) "0" "") (itoa (fix MM)) ":"
            (if (< (fix SS) 10) "0" "") (itoa (fix SS)))) 
;
;
;  Listing 5. d:LOG      Log is a program that builds a time log
;                        program for activities currently being
;       q                 performed.
;
(defun C:LOG ()
  (if (null Tstart)
   (progn
     (if (null O-Name) (setq O-Name (getstring T "\nEnter your name:")))
     (setq Temp
         (if (null Proj) (getstring T "\nProject ID:")
                (getstring T (strcat "\nProject ID <" Proj "> :"))))
     (if (null Proj) (setq Proj ""))
     (if (/= "" Temp) (setq Proj Temp))
     (setq Temp
         (if (null Actv) (getstring T "\nActivity:")
                (getstring T (strcat "\nActivity <" Actv "> :"))))
     (if (/= "" Temp) (setq Actv Temp))
     (if (null Actv) (setq Actv ""))
     (setq Tstart (cdate))
     (timer)) 
   (progn
     (setq Tstart (strcat "Start: " Tstart "  Stop: " (cdate)) Elsp (timer)) 
     (if LOG-FH
        (write-line (strcat O-name ", " Proj ", " Actv "\n" Tstart
                            "  Elapsed: " (time Elsp)) LOG-FH)
        (write-line (strcat O-name ", " Proj ", " Actv "\n" Tstart
                            "  Elapsed: " (time Elsp)))) 
      (setq Tstart nil Elsp nil))))
;
(defun C:Log-On ()
   (setq FN (getstring "\nLog file name <LOG.LOG>:"))
   (if (= FN "") (setq FN "LOG.LOG"))
   (setq LOG-FH (open FN "w"))
   (setq FN "   Log File Opened.  Type LOG to start clock."))
;
(defun C:Log-Off ()
   (setq LOG-FH (close LOG-FH))
   (prompt "\nEnding current drawing session.")
   (command "END"))

(defun c:setauto ()
  (setq answer (strcase (getstring "\nDo you want to do AUTOSAVES ?  ")))
  (if (> (strlen answer) 1)
    (setq answer (substr answer 1 1))
  )
  (if (= answer "Y")
    (progn
      (prompt "\nAUTOSAVE is on.")
      (print)
      (setq minutes (getreal "\nHow often would you like to AUTOSAVE, in minutes <15>: "))
      (if (= minutes nil)
        (setq minutes 15.0)
      )
    )
    (prompt "\nAUTOSAVE is off.")
  )
  (print)
)
(defun C:autosave ()
  (if (= answer "Y")
    (saveit)
    (prompt "\nAUTOSAVE is turned off !")
  )
  (print)
)
(defun saveit ()
  (if (= D1 nil)
    (setq D1 (getvar "DATE"))
  )
  (if (> (* (- (getvar "DATE") D1) 1440.0) minutes)
    (progn
      (setvar "cmdecho" 0)
      (prompt (strcat "\nAUTOSAVING:  " (getvar "dwgname")))
      (prompt "\nPlease stand by .... ")
      (print)
      (command "SAVE" "")
      (prompt "\nAUTOSAVE complete.")
      (prompt "\nThank you for waiting.")
      (setq D1 (getvar "DATE"))
      (setvar "cmdecho" 1)
    )
    (progn
      (prompt "\nNot time yet .... ")
      (princ (rtos (- minutes (* (- (getvar "DATE") D1) 1440.0)) 2 1))
      (princ " minutes left.")
    )
  )
  (print)
)
  "C:SETAUTO"